@@ -1,23 +1,26 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright  2002-2004 MySQL AB, 2008 Sun Microsystems
+ All rights reserved. Use is subject to license terms.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of version 2 of the GNU General Public License as 
- published by the Free Software Foundation.
+  The MySQL Connector/J is licensed under the terms of the GPL,
+  like most MySQL Connectors. There are special exceptions to the
+  terms and conditions of the GPL as it is applied to this software,
+  see the FLOSS License Exception available on mysql.com.
 
- There are special exceptions to the terms and conditions of the GPL 
- as it is applied to this software. View the full text of the 
- exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
- software distribution.
+  This program is free software; you can redistribute it and/or
+  modify it under the terms of the GNU General Public License as
+  published by the Free Software Foundation; version 2 of the
+  License.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,  
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+  02110-1301 USA
 
 
 
@@ -28,11 +31,11 @@ import java.sql.Date;
 import java.sql.SQLException;
 import java.sql.Time;
 import java.sql.Timestamp;
-
 import java.util.Calendar;
 import java.util.Collections;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.Map;
 import java.util.TimeZone;
 
@@ -117,8 +120,7 @@ public class TimeUtil {
 		tempMap.put("Canada Central Standard Time", "America/Regina");
 		tempMap.put("US Mountain", "America/Phoenix");
 		tempMap.put("US Mountain Standard Time", "America/Phoenix");
-		tempMap.put("GMT", "Europe/London");
-		tempMap.put("GMT Standard Time", "Europe/London");
+		tempMap.put("GMT", "GMT");
 		tempMap.put("Ekaterinburg", "Asia/Yekaterinburg");
 		tempMap.put("Ekaterinburg Standard Time", "Asia/Yekaterinburg");
 		tempMap.put("West Asia", "Asia/Karachi");
@@ -205,6 +207,17 @@ public class TimeUtil {
 		tempMap.put("Greenwich Standard Time", "GMT");
 		tempMap.put("UTC", "GMT");
 
+		// MySQL understands the Continent/City/region as well
+		Iterator entries = tempMap.entrySet().iterator();
+		Map entryMap = new HashMap(tempMap.size()); // to avoid ConcurrentModificationException
+		
+		while (entries.hasNext()) {
+			String name = ((Map.Entry)entries.next()).getValue().toString();
+			entryMap.put(name, name);
+		}
+		
+		tempMap.putAll(entryMap);
+		
 		TIMEZONE_MAPPINGS = Collections.unmodifiableMap(tempMap);
 
 		//
@@ -974,7 +987,8 @@ public class TimeUtil {
 		}
 		
 		dateCal.clear();
-
+		dateCal.set(Calendar.MILLISECOND, 0);
+		
 		// why-oh-why is this different than java.util.date,
 		// in the year part, but it still keeps the silly '0'
 		// for the start month????
@@ -991,25 +1005,51 @@ public class TimeUtil {
 
 		return new Date(dateAsMillis);
 	}
+	
+	final static Date fastDateCreate(int year, int month, int day, Calendar targetCalendar) {
+ 		
+		
+		Calendar dateCal = (targetCalendar == null) ? new GregorianCalendar() : targetCalendar;
+		
+		dateCal.clear();
+
+		
+		// why-oh-why is this different than java.util.date,
+		// in the year part, but it still keeps the silly '0'
+		// for the start month????
+		dateCal.set(year, month - 1, day, 0, 0, 0);
+		dateCal.set(Calendar.MILLISECOND, 0);
+		
+		long dateAsMillis = 0;
+
+		try {
+			dateAsMillis = dateCal.getTimeInMillis();
+		} catch (IllegalAccessError iae) {
+			// Must be on JDK-1.3.1 or older....
+			dateAsMillis = dateCal.getTime().getTime();
+		}
+
+		return new Date(dateAsMillis);
+	}
 
 	final static Time fastTimeCreate(Calendar cal, int hour, int minute,
-			int second) throws SQLException {
-		if (hour < 0 || hour > 23) {
+			int second, ExceptionInterceptor exceptionInterceptor) throws SQLException {
+		if (hour < 0 || hour > 24) {
 			throw SQLError.createSQLException("Illegal hour value '" + hour + "' for java.sql.Time type in value '"
 					+ timeFormattedString(hour, minute, second) + ".", 
-					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, exceptionInterceptor);
 		}
 		
 		if (minute < 0 || minute > 59) {
 			throw SQLError.createSQLException("Illegal minute value '" + minute + "'" + "' for java.sql.Time type in value '"
 					+ timeFormattedString(hour, minute, second) + ".", 
-					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, exceptionInterceptor);
 		}
 		
 		if (second < 0 || second > 59) {
 			throw SQLError.createSQLException("Illegal minute value '" + second + "'" + "' for java.sql.Time type in value '"
 					+ timeFormattedString(hour, minute, second) + ".", 
-					SQLError.SQL_STATE_ILLEGAL_ARGUMENT);
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, exceptionInterceptor);
 		}
 		
 		cal.clear();
@@ -1029,6 +1069,44 @@ public class TimeUtil {
 		return new Time(timeAsMillis);
 	}
 
+	final static Time fastTimeCreate(int hour, int minute,
+ 			int second, Calendar targetCalendar, ExceptionInterceptor exceptionInterceptor) throws SQLException {
+		if (hour < 0 || hour > 23) {
+			throw SQLError.createSQLException("Illegal hour value '" + hour + "' for java.sql.Time type in value '"
+					+ timeFormattedString(hour, minute, second) + ".", 
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, exceptionInterceptor);
+		}
+		
+		if (minute < 0 || minute > 59) {
+			throw SQLError.createSQLException("Illegal minute value '" + minute + "'" + "' for java.sql.Time type in value '"
+					+ timeFormattedString(hour, minute, second) + ".", 
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, exceptionInterceptor);
+		}
+		
+		if (second < 0 || second > 59) {
+			throw SQLError.createSQLException("Illegal minute value '" + second + "'" + "' for java.sql.Time type in value '"
+					+ timeFormattedString(hour, minute, second) + ".", 
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, exceptionInterceptor);
+		}
+		
+		Calendar cal = (targetCalendar == null) ? new GregorianCalendar() : targetCalendar;
+		cal.clear();
+
+		// Set 'date' to epoch of Jan 1, 1970
+		cal.set(1970, 0, 1, hour, minute, second);
+
+		long timeAsMillis = 0;
+
+		try {
+			timeAsMillis = cal.getTimeInMillis();
+		} catch (IllegalAccessError iae) {
+			// Must be on JDK-1.3.1 or older....
+			timeAsMillis = cal.getTime().getTime();
+		}
+
+		return new Time(timeAsMillis);
+ 	}
+	
 	final static Timestamp fastTimestampCreate(boolean useGmtConversion,
 			Calendar gmtCalIfNeeded,
 			Calendar cal, int year,
@@ -1059,6 +1137,38 @@ public class TimeUtil {
 			offsetDiff = fromOffset - toOffset;
 		}
 
+		if (secondsPart != 0) {
+			cal.set(Calendar.MILLISECOND, secondsPart / 1000000);
+		}
+		
+		long tsAsMillis = 0;
+
+		
+		try {
+			tsAsMillis = cal.getTimeInMillis();
+		} catch (IllegalAccessError iae) {
+			// Must be on JDK-1.3.1 or older....
+			tsAsMillis = cal.getTime().getTime();
+		}
+
+		Timestamp ts = new Timestamp(tsAsMillis + offsetDiff);
+		
+		ts.setNanos(secondsPart);
+
+		return ts;
+	}
+	
+	final static Timestamp fastTimestampCreate(TimeZone tz, int year,
+ 			int month, int day, int hour, int minute, int seconds,
+ 			int secondsPart) {
+		Calendar cal = (tz == null) ? new GregorianCalendar() : new GregorianCalendar(tz);
+		cal.clear();
+		
+		// why-oh-why is this different than java.util.date,
+		// in the year part, but it still keeps the silly '0'
+		// for the start month????
+		cal.set(year, month - 1, day, hour, minute, seconds);
+
 		long tsAsMillis = 0;
 
 		try {
@@ -1068,12 +1178,12 @@ public class TimeUtil {
 			tsAsMillis = cal.getTime().getTime();
 		}
 
-		Timestamp ts = new Timestamp(tsAsMillis + offsetDiff);
+		Timestamp ts = new Timestamp(tsAsMillis);
 		ts.setNanos(secondsPart);
 
 		return ts;
 	}
-
+	
 	/**
 	 * Returns the 'official' Java timezone name for the given timezone
 	 * 
@@ -1081,17 +1191,26 @@ public class TimeUtil {
 	 *            the 'common' timezone name
 	 * 
 	 * @return the Java timezone name for the given timezone
+	 * @throws SQLException 
 	 * 
 	 * @throws IllegalArgumentException
 	 *             DOCUMENT ME!
 	 */
-	public static String getCanoncialTimezone(String timezoneStr) {
+	public static String getCanoncialTimezone(String timezoneStr, ExceptionInterceptor exceptionInterceptor) throws SQLException {
 		if (timezoneStr == null) {
 			return null;
 		}
 
 		timezoneStr = timezoneStr.trim();
 
+		// handle '+/-hh:mm' form ...
+		
+		if (timezoneStr.length() > 2) {
+			if ((timezoneStr.charAt(0) == '+' || timezoneStr.charAt(0) == '-') &&
+					Character.isDigit(timezoneStr.charAt(1))) {
+				return "GMT" + timezoneStr;
+			}
+		}
 		// Fix windows Daylight/Standard shift JDK doesn't map these (doh)
 
 		int daylightIndex = StringUtils.indexOfIgnoreCase(timezoneStr,
@@ -1118,26 +1237,17 @@ public class TimeUtil {
 				if (abbreviatedTimezone.length == 1) {
 					canonicalTz = abbreviatedTimezone[0];
 				} else {
-					StringBuffer errorMsg = new StringBuffer(
-							"The server timezone value '");
-					errorMsg.append(timezoneStr);
-					errorMsg
-							.append("' represents more than one timezone. You must ");
-					errorMsg
-							.append("configure either the server or client to use a ");
-					errorMsg
-							.append("more specifc timezone value if you want to enable ");
-					errorMsg.append("timezone support. The timezones that '");
-					errorMsg.append(timezoneStr);
-					errorMsg.append("' maps to are: ");
-					errorMsg.append(abbreviatedTimezone[0]);
-
+					StringBuffer possibleTimezones = new StringBuffer(128);
+					
+					possibleTimezones.append(abbreviatedTimezone[0]);
+					
 					for (int i = 1; i < abbreviatedTimezone.length; i++) {
-						errorMsg.append(", ");
-						errorMsg.append(abbreviatedTimezone[i]);
+						possibleTimezones.append(", ");
+						possibleTimezones.append(abbreviatedTimezone[i]);
 					}
 
-					throw new IllegalArgumentException(errorMsg.toString());
+					throw SQLError.createSQLException(Messages.getString("TimeUtil.TooGenericTimezoneId",
+							new Object[] {timezoneStr, possibleTimezones}), SQLError.SQL_STATE_INVALID_CONNECTION_ATTRIBUTE, exceptionInterceptor);
 				}
 			}
 		}
