@@ -1,23 +1,26 @@
 /*
- Copyright (C) 2002-2004 MySQL AB
+ Copyright  2002-2004 MySQL AB, 2008 Sun Microsystems
+ All rights reserved. Use is subject to license terms.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of version 2 of the GNU General Public License as 
- published by the Free Software Foundation.
+  The MySQL Connector/J is licensed under the terms of the GPL,
+  like most MySQL Connectors. There are special exceptions to the
+  terms and conditions of the GPL as it is applied to this software,
+  see the FLOSS License Exception available on mysql.com.
 
- There are special exceptions to the terms and conditions of the GPL 
- as it is applied to this software. View the full text of the 
- exception in file EXCEPTIONS-CONNECTOR-J in the directory of this 
- software distribution.
+  This program is free software; you can redistribute it and/or
+  modify it under the terms of the GNU General Public License as
+  published by the Free Software Foundation; version 2 of the
+  License.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,  
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+  02110-1301 USA
 
 
  
@@ -31,6 +34,7 @@ import java.lang.reflect.Proxy;
 import java.sql.SQLException;
 import java.util.Map;
 
+import com.mysql.jdbc.ExceptionInterceptor;
 import com.mysql.jdbc.SQLError;
 
 /**
@@ -38,7 +42,7 @@ import com.mysql.jdbc.SQLError;
  * 
  * @author Mark matthews
  * 
- * @version $Id: WrapperBase.java 6631 2007-10-11 20:01:16Z mmatthews $
+ * @version $Id$
  */
 abstract class WrapperBase {
 	protected MysqlPooledConnection pooledConnection;
@@ -65,7 +69,13 @@ abstract class WrapperBase {
 	}
 	
 	protected Map unwrappedInterfaces = null;
+	protected ExceptionInterceptor exceptionInterceptor;
 
+	protected WrapperBase(MysqlPooledConnection pooledConnection) {
+		this.pooledConnection = pooledConnection;
+		this.exceptionInterceptor = this.pooledConnection.getExceptionInterceptor();
+	}
+	
 	protected class ConnectionErrorFiringInvocationHandler implements InvocationHandler {
 		Object invokeOn = null;
 		
